/*=======================================================================
  Module: demo 
=======================================================================*/ 

set sqlblanklines on

----------------------------------------------------------------------------
-- cleanup
----------------------------------------------------------------------------
begin
  -- delete module if exists
  ords.delete_module( p_module_name => 'demo');
end;
/



declare
  l_moduleid   number;
  l_templateid number;
  l_handlerid  number;
  l_module_name user_ords_modules.name%type := 'demo';
  l_pattern user_ords_templates.uri_template%type;
begin
  -- delete module if exists
  ords.delete_module( p_module_name => l_module_name);
  
  -- create module
  ords.define_module(
    p_module_name       => l_module_name
   ,p_base_path => 'demo/'
   ,p_status     => 'PUBLISHED'
  );

  ----------------------------------------------------------------------------
  -- RESOURCE TEMPLATE departments/ 
  ----------------------------------------------------------------------------
  l_pattern := 'departments/';
  ords.define_template(
    p_module_name    => l_module_name
   ,p_pattern => l_pattern
  );

  --------------------------------------
  -- HANDLER GET departments/ 
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_collection_feed
   ,p_source       => q'[
   
   select deptno "$self", 
          dept.* 
     from dept
   
   ]'
   ,p_method       => 'GET'
   ,p_items_per_page => 100
  );  

  --------------------------------------
  -- HANDLER POST departments/ 
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_plsql
   ,p_source       => q'[
   
    begin
      insert into dept(deptno, dname, loc)
        values (:deptno, :dname, :loc);
      
      owa_util.mime_header('application/json', true);
      htp.p('{"status": "created", "deptno": '||:deptno ||'}');
      
      -- set http return code 201: CREATED
      :status := 201;
    end;

   ]'
   ,p_method       => 'POST'
   ,p_items_per_page => 100
  );  
  
  ORDS.DEFINE_PARAMETER(
   p_module_name         => l_module_name,
   p_pattern             => l_pattern,
   p_method              => 'POST',
   p_name                => 'X-APEX-STATUS-CODE',
   p_bind_variable_name  => 'status',
   p_source_type         => 'HEADER', -- header, response, uri
   p_param_type          => 'INT',
   p_access_method       => 'OUT',
   p_comments            => '' );

  ----------------------------------------------------------------------------
  -- RESOURCE TEMPLATE departments/{deptno} 
  ----------------------------------------------------------------------------
  l_pattern := 'departments/{deptno}';
  ords.define_template(
    p_module_name    => l_module_name
   ,p_pattern => l_pattern
  );

  --------------------------------------
  -- GET departments/{deptno}
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_collection_item
   ,p_source       => q'[
   
   select deptno "$self", 
          dept.* ,
          deptno || '/employees/' "$employees"
     from dept
    where deptno=to_number(:deptno)
   
   ]'
   ,p_method       => 'GET'
   ,p_items_per_page => 100
  );  
  
  --------------------------------------
  -- PUT departments/{deptno}
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_plsql
   ,p_source       => q'[
   
   declare
     l_num_rows pls_integer;
   begin
     update dept
        set dname=:dname,
            loc=:loc
      where deptno=to_number(:deptno);
      
      l_num_rows := sql%rowcount;
      
      owa_util.mime_header('application/json', true);
      htp.p('{"status": "updated", "deptno": '||:deptno ||', "numRows": '|| l_num_rows ||'}');
   end;
   
   ]'
   ,p_method       => 'PUT'
   ,p_items_per_page => 100
  );  

  

  --------------------------------------
  -- DELETE departments/{deptno}
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_plsql
   ,p_source       => q'[
   
   begin
    delete 
     from dept 
    where deptno=to_number(:deptno);
   end;
   
   ]'
   ,p_method       => 'DELETE'
   ,p_items_per_page => 100
  );  

  ----------------------------------------------------------------------------
  -- RESOURCE TEMPLATE departments/{deptno}/employees 
  ----------------------------------------------------------------------------
  l_pattern := 'departments/{deptno}/employees/';
  ords.define_template(
    p_module_name    => l_module_name
   ,p_pattern => l_pattern
  );

  --------------------------------------
  -- GET departments/{deptno}/employees
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_collection_feed
   ,p_source       => q'[
   
   select emp.*,
          '../../' || emp.deptno "$department"
     from emp
    where deptno=to_number(:deptno)
   
   ]'
   ,p_method       => 'GET'
   ,p_items_per_page => 100
  ); 
  
  ----------------------------------------------------------------------------
  -- RESOURCE TEMPLATE user-info 
  ----------------------------------------------------------------------------
  l_pattern := 'user-info';
  ords.define_template(
    p_module_name    => l_module_name
   ,p_pattern => l_pattern
  );

  --------------------------------------
  -- GET departments/{deptno}/employees
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_collection_feed
   ,p_source       => q'[
   
      SELECT SYS_CONTEXT ('USERENV', 'CURRENT_SCHEMA') current_schema,
             SYS_CONTEXT ('USERENV', 'PROXY_USER') proxy_user,
             SYS_CONTEXT ('USERENV', 'SESSION_USER') session_user,
             SYS_CONTEXT ('USERENV', 'SESSIONID') session_id,
             SYS_CONTEXT ('USERENV', 'SID') sid,
             USER
        FROM DUAL
   
   ]'
   ,p_method       => 'GET'
   ,p_items_per_page => 100
  );  

  ----------------------------------------------------------------------------
  -- RESOURCE TEMPLATE employees/ 
  ----------------------------------------------------------------------------
  l_pattern := 'employees/';
  ords.define_template(
    p_module_name    => l_module_name
   ,p_pattern => l_pattern
  );

  --------------------------------------
  -- GET employees/ 
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_collection_feed
   ,p_source       => q'[
   
   select empno "$self", 
          emp.* 
     from emp
   
   ]'
   ,p_method       => 'GET'
   ,p_items_per_page => 100
  );  

  ----------------------------------------------------------------------------
  -- RESOURCE TEMPLATE employees/{empno}
  ----------------------------------------------------------------------------
  l_pattern := 'employees/{empno}';
  ords.define_template(
    p_module_name    => l_module_name
   ,p_pattern => l_pattern
  );

  --------------------------------------
  -- GET employees/{empno}
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_collection_item
   ,p_source       => q'[
   
   select empno "$self", 
          emp.*
     from emp
    where empno=to_number(:empno)
   
   ]'
   ,p_method       => 'GET'
   ,p_items_per_page => 100
  );  
 
  --------------------------------------
  -- PUT employees/{empno}
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_plsql
   ,p_source       => q'[
   
    declare
      l_msg varchar2(4000);
      l_scope logger_logs.scope%type := 'PUT employees/{empno}';
      l_params logger.tab_param;
      
    begin
      -- URL parameter (implicit)
      logger.append_param(l_params, 'empno', :empno);
      
      -- Regular http header variables (explicit)
      logger.append_param(l_params, 'user_agent', :user_agent);
      
      -- Request (implicit)
      logger.append_param(l_params, 'content_type', :content_type);
      logger.append_param(l_params, 'body', wwv_flow_utilities.blob_to_clob(:body));
      
      logger.log('START', l_scope, null, l_params);
    
      -- set JSON return declaratively
      -- this one will be returned as JSON
      -- (explicit)
      :status := 'updated';
      :empno := 99;
      
      logger.log('END', l_scope);
    end;

   ]'
   ,p_method       => 'PUT'
   ,p_items_per_page => 100
  );  

  ORDS.DEFINE_PARAMETER(
   p_module_name         => l_module_name,
   p_pattern             => l_pattern,
   p_method              => 'PUT',
   p_name                => 'Status',
   p_bind_variable_name  => 'status',
   p_source_type         => 'URI', -- header, response, uri
   p_param_type          => 'STRING',
   p_access_method       => 'OUT',
   p_comments            => '' );
   
  ORDS.DEFINE_PARAMETER(
   p_module_name         => l_module_name,
   p_pattern             => l_pattern,
   p_method              => 'PUT',
   p_name                => 'User-Agent',
   p_bind_variable_name  => 'user_agent',
   p_source_type         => 'HEADER', -- header, response, uri
   p_param_type          => 'STRING',
   p_access_method       => 'IN',
   p_comments            => '' );

  ORDS.DEFINE_PARAMETER(
   p_module_name         => l_module_name,
   p_pattern             => l_pattern,
   p_method              => 'PUT',
   p_name                => 'empno',
   p_bind_variable_name  => 'empno',
   p_source_type         => 'URI', -- HEADER, RESPONSE, URI
   p_param_type          => 'INT', -- STRING, INT, DOUBLE, BOOLEAN, LONG, TIMESTAMP.
   p_access_method       => 'INOUT', -- IN, OUT, INOUT
   p_comments            => '' );

  ----------------------------------------------------------------------------
  -- RESOURCE TEMPLATE cgi-env
  ----------------------------------------------------------------------------
  l_pattern := 'cgi-env';
  ords.define_template(
    p_module_name    => l_module_name
   ,p_pattern => l_pattern
  );

  --------------------------------------
  -- GET cgi-env 
  --------------------------------------
  ords.define_handler(
    p_module_name       => l_module_name
   ,p_pattern => l_pattern
   ,p_source_type  => ords.source_type_plsql
   ,p_source       => q'[
   
      begin
        htp.p('user: '||owa_util.get_cgi_env('REMOTE_USER'));
        htp.p('<br><br><br>');
        owa_util.print_cgi_env;
      end;

   ]'
   ,p_method       => 'GET'
   ,p_items_per_page => 100
  );  

end;
/


/*
  source_type_query           CONSTANT ords_handlers.source_type%type := 'json/query';
  source_type_csv_query       CONSTANT ords_handlers.source_type%type := 'csv/query';
  source_type_query_one_row   CONSTANT ords_handlers.source_type%type := 'json/query;type=single';
  source_type_feed            CONSTANT ords_handlers.source_type%type := 'json/query;type=feed';
  source_type_media           CONSTANT ords_handlers.source_type%type := 'resource/lob';
  source_type_plsql           CONSTANT ords_handlers.source_type%type := 'plsql/block';
  source_type_collection_feed CONSTANT ords_handlers.source_type%type := 'json/collection';
  source_type_collection_item CONSTANT ords_handlers.source_type%type := 'json/item';
*/


commit
/
/*


select * from all_objects where owner='ORDS_METADATA'
and object_type='VIEW';

select * from user_ords_services;
select * from all_ords_services;

select * from user_ords_modules;
select * from all_ords_modules;


*/